//--------------------------------------------------------------------------------------
//	$Id: no-password-save.js,v 1.1 2022/11/08 22:11:29 wolf Exp $
//--------------------------------------------------------------------------------------
//	Copyright (c) 2022 Wolfgang Stanglmeier <software@netzwolf.info>
//	This program is free software. It comes without any warranty.
//	You can redistribute it and/or modify it under the terms of the
//	"Do What The Fuck You Want To Public License" (WTFPL) Version 2,
//	as published by Sam Hocevar. See http://www.wtfpl.net/
//--------------------------------------------------------------------------------------
//	Prevent "save password" dialog for password fields marked 'autocomplete="off"'
//--------------------------------------------------------------------------------------
//	(Mozilla provides no way to prevent the save password dialog.)
//	(I hate the arrogant morons at Mozilla.)
//--------------------------------------------------------------------------------------
//	Load script using "defer" or at end of document body
//--------------------------------------------------------------------------------------

/*jsl:option explicit*/

'use strict';

(function(){

	//------------------------------------------------------------------------------
	//	install "save password" protection on password field
	//------------------------------------------------------------------------------

	function install( password_field ){

		//----------------------------------------------------------------------
		//	data-loglevel="debug" attribute enables logging to console.debug
		//----------------------------------------------------------------------

		const log = console[
				password_field.getAttribute( 'data-loglevel' ) || ''];

		//----------------------------------------------------------------------
		//	log
		//----------------------------------------------------------------------

		log && log( 'install( ' + password_field + ').' );

		//----------------------------------------------------------------------
		//	hidden field
		//----------------------------------------------------------------------

		var hidden_field = null;

		//----------------------------------------------------------------------
		//	copy password data to hidden field then unset password field
		//----------------------------------------------------------------------

		function unset(){

			if( hidden_field ){ return; }

			log && log( '(unset)' );

			//--------------------------------------------------------------
			//	add hidden field
			//--------------------------------------------------------------

			hidden_field = document.createElement( 'input' );
			hidden_field.type = 'hidden';

			//--------------------------------------------------------------
			//	copy password data to hidden field
			//--------------------------------------------------------------

			hidden_field.name  = password_field.name;
			hidden_field.value = password_field.value;

			//--------------------------------------------------------------
			//	hidden field must *not* be appended after password field
			//--------------------------------------------------------------

			password_field.form.insertBefore( hidden_field,
						password_field.form.firstChild );

			//--------------------------------------------------------------
			//	unset password field
			//--------------------------------------------------------------
			//	value *must be cleared* (removing name is *not* enough)
			//--------------------------------------------------------------

			password_field.name  = '';
			password_field.value = '';

			//--------------------------------------------------------------
			//	log
			//--------------------------------------------------------------

			if( log ){
				log( 'password_field.typ: ' + password_field.type );
				log( 'password_field.nam: ' + password_field.name );
				log( 'password_field.val: ' + password_field.value );
				log( 'hidden_field.typ: ' + hidden_field.type );
				log( 'hidden_field.nam: ' + hidden_field.name );
				log( 'hidden_field.val: ' + hidden_field.value );
			}
		}

		//----------------------------------------------------------------------
		//	restore password data from hidden field then delete hidden field
		//----------------------------------------------------------------------

		function restore(){

			if( !hidden_field ){ return; }

			log && log( '(restore)' );

			//--------------------------------------------------------------
			//	restore password data from hidden field
			//--------------------------------------------------------------

			password_field.name  = hidden_field.name;
			password_field.value = hidden_field.value;

			//--------------------------------------------------------------
			//	log
			//--------------------------------------------------------------

			if( log ){
				log( 'password_field.type: ' + password_field.type );
				log( 'password_field.name: ' + password_field.name );
				log( 'password_field.valu: ' + password_field.value );
			}

			//--------------------------------------------------------------
			//	delete hidden field
			//--------------------------------------------------------------

			hidden_field.name  = '';
			hidden_field.value = '';
			hidden_field.parentElement.removeChild( hidden_field );

			hidden_field = null;
		}

		//----------------------------------------------------------------------
		//	restore password data on focus 
		//----------------------------------------------------------------------

		password_field.addEventListener( 'focus', restore );

		//----------------------------------------------------------------------
		//	unset data on press/touch submit fields/buttons
		//----------------------------------------------------------------------

		const elements = password_field.form.elements;

		for( var i=0 ; i < elements.length ; ++i ){

			const element = elements[ i ];
			if( element.type != 'submit' ){ continue; }

			log && log( 'add submit listener(' + element + ').' );

			element.addEventListener( 'mousedown',  unset );
			element.addEventListener( 'touchstart', unset );
		}
	}

	//------------------------------------------------------------------------------
	//	lookup autocomplete="off" password fields and install handlers
	//------------------------------------------------------------------------------

	const password_fields = document.querySelectorAll(
					'input[type="password"][autocomplete="off"' );

	for( var i=0 ; i < password_fields.length ; ++i ){

		install( password_fields[ i ]);
	}
})();

//--------------------------------------------------------------------------------------
//	$Id: no-password-save.js,v 1.1 2022/11/08 22:11:29 wolf Exp $
//--------------------------------------------------------------------------------------
